@@ -2,10 +2,30 @@ package threads::lite::tid;
 
 use strict;
 use warnings;
+use Scalar::Util qw/blessed/;
+
+use overload '~~' => sub {
+	my ($self, $other, $reverse) = @_;
+	if (blessed($other) && $other->isa(__PACKAGE__)) {
+		return $self->id == $other->id;
+	}
+	else {
+		return $self eq $other;
+	}
+  },
+  '""' => sub {
+	my $self = shift;
+	return "thread=${$self}";
+  },
+  'eq' => sub {
+	my ($self, $other, $reverse) = @_;
+	($self, $other) = ($other, $self) if $reverse;
+	return "$self" ~~ $other;
+  };
 
 use threads::lite qw/self receive/;
 
-our $VERSION = $threads::lite::VERSION;
+our $VERSION = 0.029_001;
 
 sub rpc {
 	my ($self, @arguments) = @_;
@@ -29,7 +49,7 @@ threads::lite::tid - a threads::lite thread id
 
 =head1 VERSION
 
-Version 0.010
+Version 0.029_001
 
 =head1 SYNOPSIS
 
@@ -41,13 +61,13 @@ This module represents a thread ID object. It provides a handle to a thread.
 
 Send a message to a thread. The message items may contain any data type that can be serialized by L<Storable> (including coderefs).
 
-=head2 id
+=head2 id()
 
-Get the numerical id of this thread.
+Get an opaque but primitive identifier for thread.
 
-=head2 monitor
+=head2 monitor()
 
-Monitor the thread. This will cause the calling thread to get a notification of the thread's end. In case of a natural death it returns C<('normal', @return_value)>, in case of an unnatural death it will contain C<('error', $exception)>.
+Monitor the thread. This will cause the calling thread to get a notification of the thread's end. In case of a natural death it returns C<('exit', 'normal', $id, @return_value)>, in case of an unnatural death it will contain C<('exit', 'error', $id, $exception)>.
 
 =head2 rpc(@args)
 
@@ -65,16 +85,12 @@ Please report any bugs or feature requests to C<bug-threads-lite at rt.cpan.org>
 the web interface at L<http://rt.cpan.org/NoAuth/ReportBug.html?Queue=threads-lite>.  I will be notified, and then you'll
 automatically be notified of progress on your bug as I make changes.
 
-
-
-
 =head1 SUPPORT
 
 You can find documentation for this module with the perldoc command.
 
     perldoc threads::lite
 
-
 You can also look for information at:
 
 =over 4
@@ -97,21 +113,12 @@ L<http://search.cpan.org/dist/threads-lite>
 
 =back
 
-
-=head1 ACKNOWLEDGEMENTS
-
-
 =head1 COPYRIGHT & LICENSE
 
-Copyright 2009 Leon Timmermans, all rights reserved.
+Copyright 2009, 2010 Leon Timmermans, all rights reserved.
 
 This program is free software; you can redistribute it and/or modify it
 under the same terms as Perl itself.
 
 =cut
 
-=for ignore
-
-=head2 STORABLE_freeze
-
-=cut
