@@ -11,8 +11,7 @@
 
 static SV* S_message_get_sv(pTHX_ message* message) {
 	SV* stored = newSVpvn(message->string.ptr, message->string.length);
-	PerlMemShared_free(message->string.ptr);
-	message->string.ptr = NULL;
+	message_destroy(message);
 	return stored;
 }
 
@@ -43,12 +42,12 @@ void S_message_store_value(pTHX_ message* message, SV* value) {
 	PUTBACK;
 }
 
-static inline int S_is_simple(pTHX_ SV* value) {
+static int S_is_simple(pTHX_ SV* value) {
 	return SvOK(value) && !SvROK(value) && !(SvPOK(value) && SvUTF8(value));
 }
 #define is_simple(value) S_is_simple(aTHX_ value)
 
-static inline int S_are_simple(pTHX_ SV** begin, SV** end) {
+static int S_are_simple(pTHX_ SV** begin, SV** end) {
 	SV** current;
 	for(current = begin; current <= end; current++)
 		if (! is_simple(*current))
@@ -78,6 +77,7 @@ void S_message_from_stack(pTHX_ message* message) {
 
 SV* S_message_load_value(pTHX_ message* message) {
 	dSP;
+	SV* ret;
 
 	sv_setiv(save_scalar(gv_fetchpv("Storable::Eval", TRUE | GV_ADDMULTI, SVt_PV)), 1);
 	PUSHMARK(SP);
@@ -85,7 +85,7 @@ SV* S_message_load_value(pTHX_ message* message) {
 	PUTBACK;
 	call_pv("Storable::thaw", G_SCALAR);
 	SPAGAIN;
-	SV* ret = POPs;
+	ret = POPs;
 	PUTBACK;
 	return ret;
 }
@@ -97,7 +97,7 @@ void S_message_to_stack(pTHX_ message* message, U32 context) {
 			PUSHs(sv_2mortal(newRV_noinc(message_get_sv(message))));
 			break;
 		case PACKED: {
-			SV* mess = message_get_sv(message);
+			SV* mess = sv_2mortal(message_get_sv(message));
 			STRLEN len;
 			const char* packed = SvPV(mess, len);
 			PUTBACK;
@@ -128,40 +128,39 @@ void S_message_to_stack(pTHX_ message* message, U32 context) {
 	PUTBACK;
 }
 
-void S_message_to_array(pTHX_ message* message, AV** array_ptr) {
+AV* S_message_to_array(pTHX_ message* message) {
 	dSP;
+	AV* ret;
 	switch(message->type) {
 		case STRING:
-			*array_ptr = newAV();
-			av_push(*array_ptr, message_get_sv(message));
-			sv_2mortal((SV*)*array_ptr);
+			av_create_and_push(&ret, message_get_sv(message));
 			break;
 		case PACKED: {
 			SV* mess = message_get_sv(message);
 			STRLEN len;
+			int count;
 			const char* packed = SvPV(mess, len);
 			SV** mark = SP;
 			PUTBACK;
-			int count = unpackstring(pack_template, pack_template + sizeof pack_template - 1, packed, packed + len, 0);
+			count = unpackstring(pack_template, pack_template + sizeof pack_template - 1, packed, packed + len, 0);
 			SPAGAIN;
-			*array_ptr = av_make(count, mark + 1);
-			sv_2mortal((SV*)*array_ptr);
+			ret = av_make(count, mark + 1);
 			break;
 		}
 		case STORABLE: {
-			*array_ptr = (AV*) SvRV(message_load_value(message));
+			ret = (AV*)SvREFCNT_inc(SvRV(message_load_value(message)));
 			SPAGAIN;
 			break;
 		}
 		default:
 			Perl_croak(aTHX_ "Type %d is not yet implemented", message->type);
 	}
-
 	PUTBACK;
+
+	return ret;
 }
 
 void S_message_clone(pTHX_ message* origin, message* clone) {
-	clone->type = origin->type;
 	switch (origin->type) {
 		case EMPTY:
 			Perl_croak(aTHX_ "Empty messages aren't allowed yet\n");
@@ -169,6 +168,7 @@ void S_message_clone(pTHX_ message* origin, message* clone) {
 		case STRING:
 		case PACKED:
 		case STORABLE:
+			clone->type = origin->type;
 			clone->string.length = origin->string.length;
 			clone->string.ptr = savesharedpvn(origin->string.ptr, origin->string.length);
 			break;
